﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityStandardAssets.Characters.ThirdPerson;

public class LevelMode : Assets.Scripts.StateComponent {

    public static LevelMode CURRENT;

    public const string STATENAME_SETMODE = "setmode";
    public const string STATENAME_GAMEMODE = "gamemode";

    public static bool InSetMode()
    {
        if (CURRENT == null) {
            return false;
        }
        return CURRENT.GetState().GetName() == STATENAME_SETMODE;
    }

    public static bool InGameMode()
    {
        if (CURRENT == null) {
            return false;
        }
        return CURRENT.GetState().GetName() == STATENAME_GAMEMODE;
    }

    public static void restart()
    {
        Debug.Log("RESTART!!!");
        SceneManager.LoadScene(SceneManager.GetActiveScene().buildIndex); 
    }


    public class SetMode : Assets.Scripts.State
    {
        LevelMode lMode;
        GameObject selected = null;
        Vector3 startVec;
        bool dragged = false;

        public override void OnEnter()
        {
            Debug.Log("ENTER SETMODE");
            LevelMode.CURRENT.gameCam.enabled = false;
            LevelMode.CURRENT.setmodeCam.enabled = true;
            LevelMode.CURRENT.player.GetComponent<ThirdPersonUserControl>().enabled = false;
            CURRENT.editCanvas.enabled = true;
            CURRENT.winCanvas.enabled = false;
            LevelMode.CURRENT.gameCanvas.enabled = false;


        }

        public override void OnLeave()
        {
            Debug.Log("EXIT SETMODE");
        }

        public override void OnUpdate()
        {
            if (Input.GetMouseButton(0))
            {
                if (!dragged)
                {
                    dragged = true;
                    dragStart();
                }
                else
                {
                    dragMove();
                }
            } else
            {
                if (dragged)
                {
                    dragEnd();
                    dragged = false;
                }
            }
        }

        public override string GetName()
        {
            return STATENAME_SETMODE;
        }

        public void dragStart()
        {
            Debug.Log("DRAG START!");
            RaycastHit hit;
            Ray ray = LevelMode.CURRENT.setmodeCam.ScreenPointToRay(Input.mousePosition);
            if (Physics.Raycast(ray, out hit, 100.0f))
            {
                Property properties = hit.collider.GetComponent<Property>();
                if (properties!=null && properties.hasProperty("moveable"))
                {
                    selected = hit.collider.gameObject;
                    startVec = selected.transform.position;
                    Debug.Log("SELECTED " + selected.name);
                } else
                {
                    Debug.Log("NOT SELECTED: " + hit.collider.name);
                }
            }
        }

        public void dragMove()
        {
            Debug.Log("DRAG MOVE!");
            if (selected != null)
            {
                RaycastHit hit;
                Ray ray = LevelMode.CURRENT.setmodeCam.ScreenPointToRay(Input.mousePosition);
                if (Physics.Raycast(ray, out hit, 100.0f,CURRENT.groundsLayer))
                {
                    Debug.Log("DM:" + hit.collider.name);
                    Property properties = hit.collider.GetComponent<Property>();
                    if (properties!=null && properties.hasProperty("drop"))
                    {
                        selected.transform.position = hit.point;
                    } else
                    {
                        Debug.Log("NO PROPS");
                    }
                }
            }
        }

        public void dragEnd()
        {
            Debug.Log("DRAG END!");
            if (selected != null)
            {
                RaycastHit hit;
                Ray ray = LevelMode.CURRENT.setmodeCam.ScreenPointToRay(Input.mousePosition);
                if (Physics.Raycast(ray, out hit, 100.0f,CURRENT.groundsLayer))
                {
                    Property properties = hit.collider.GetComponent<Property>();
                    if (properties == null || !properties.hasProperty("drop"))
                    {
                        selected.transform.position = startVec;
                    }
                }
            }
            selected = null;
        }



    }

    public class GameMode : Assets.Scripts.State
    {
        LevelMode lMode;

        public override void OnEnter()
        {
            Debug.Log("ENTER GAMEMODE");
            LevelMode.CURRENT.gameCam.enabled = true;
            LevelMode.CURRENT.setmodeCam.enabled = false;
            LevelMode.CURRENT.gameCanvas.enabled = true;

            LevelMode.CURRENT.player.GetComponent<ThirdPersonUserControl>().enabled = true;
            CURRENT.editCanvas.enabled = false;
            CURRENT.winCanvas.enabled = false;
        }

        public override void OnLeave()
        {
            Debug.Log("EXIT GAMEMODE");
        }

        public override void OnUpdate()
        {
            if (CURRENT.enemyKillsToWin == 0)
            {
                CURRENT.SetState(CURRENT.STATE_WINMODE);
            }
        }

        public override string GetName()
        {
            return STATENAME_GAMEMODE;
        }
    }

    public class WinMode : Assets.Scripts.State
    {
        LevelMode lMode;

        public override void OnEnter()
        {
            Debug.Log("ENTER WINMODE");
            LevelMode.CURRENT.gameCam.enabled = true;
            LevelMode.CURRENT.setmodeCam.enabled = false;
            LevelMode.CURRENT.gameCanvas.enabled = false;
            LevelMode.CURRENT.player.GetComponent<ThirdPersonUserControl>().enabled = false;
            CURRENT.editCanvas.enabled = false;
            CURRENT.winCanvas.enabled = true;
        }

        public override void OnLeave()
        {
            Debug.Log("EXIT SETMODE");
        }

        public override void OnUpdate()
        {

        }

        public override string GetName()
        {
            return STATENAME_GAMEMODE;
        }
    }


    public enum Mode {
        setmode,gamemode
    }

    public Mode startMode = Mode.setmode;
    public SetMode STATE_SETMODE = new SetMode();
    public GameMode STATE_GAMEMODE = new GameMode();
    public WinMode STATE_WINMODE = new WinMode();
    public int maxMooSounds = 3;
    public int currentSounds = 0;
    public Camera setmodeCam;
    public Camera gameCam;
    public GameObject player;
    public Canvas editCanvas;
    public Canvas winCanvas;
    public Canvas gameCanvas;

    public int enemyKillsToWin=0;
    public string nextScene;
    public TNTDetonator detonator;
    public Material deactivatedMaterial;


    public int groundsLayer = 0;

    public void Awake()
    {
        CURRENT = this;
        groundsLayer = LayerMask.GetMask("grounds");
    }

	// Use this for initialization
	void Start () {
        switch (startMode)
        {
            case Mode.setmode:
                SetState(STATE_SETMODE);
                break;
            case Mode.gamemode:
                SetState(STATE_GAMEMODE);
                break;
        }
    }

    public bool addMoo() {
        if (currentSounds + 1 <= maxMooSounds) {
            currentSounds++;
            return true;
        }
        return false;
    }

    public void removeMoo() {
        if (currentSounds > 0) {
            currentSounds--;
        }
    }

    public void doRestart()
    {
        restart();
    }

    public void addEnemies(int amount)
    {
        enemyKillsToWin += amount;
    }

    public void onEnemyKill()
    {
        enemyKillsToWin--;
    }

    void OnDrawGizmos()
    {
        Gizmos.color = Color.yellow;
        Gizmos.DrawSphere(transform.position, 1);
    }

    public void startLevel()
    {
        SetState(STATE_GAMEMODE);
    }

    public void nextLevel()
    {
        Debug.Log("NEXT LEVEL");
        if (GetState() != STATE_WINMODE) {
            Debug.Log("WHY?");
        } else {
            SceneManager.LoadScene(nextScene);
        }
    }

}
