﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Assets.Scripts
{
    public static class Utils
    {
        public static Transform FindDeepChild(this Transform aParent, string aName)
        {
            var result = aParent.Find(aName);
            if (result != null)
                return result;
            foreach (Transform child in aParent)
            {
                result = child.FindDeepChild(aName);
                if (result != null)
                    return result;
            }
            return null;
        }
    }

    public class State
    {
        public virtual string GetName() { return ""; }
        public virtual void OnEnter() { }
        public virtual void OnLeave() { }
        public virtual void OnUpdate() { }
    }

    public class StateComponent : MonoBehaviour
    {
        protected State currentState;        
        public void SetState(State newState)
        {
            if (newState == currentState) {
                return;
            }

            if (currentState != null)
            {
                currentState.OnLeave();
            }
            currentState = newState;
            currentState.OnEnter();
        }

        public void Update() {
            if (currentState != null)
            {
                currentState.OnUpdate();
            }
        }

        public State GetState()
        {
            return currentState;
        }
    }
}
