﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;

public class Enemy : MonoBehaviour {

    public float updateTargetDistance = 0.2f;
    public GameObject target;
    private Vector3 lastTargetPos;
    public float scaleVariance = 0.15f;
    public float baseSpeed = 2.0f;
    public float springDeathFactor = 2.0f;

    public AudioClip[] deathSounds;
    public AudioClip[] mooSounds;
    public float mooTimer;
    public float mooVariance;

    public float currentMooTimer;

    private Animator animator;

    private NavMeshAgent agent;
    private CharacterController characterController;

    private AudioSource source;

    public bool audioPlaying = false;

	// Use this for initialization
	void Start () {
        source = GetComponent<AudioSource>();
        animator = GetComponent<Animator>();
        baseSpeed += Random.Range(-0.25f, 0.25f);
        animator.speed = baseSpeed;
        characterController = GetComponent<CharacterController>();
        float scale = 1 + Random.Range( -scaleVariance,scaleVariance );
        transform.localScale = new Vector3(scale,scale,scale);

        agent = GetComponent<NavMeshAgent>();	
        if (target != null)
        {
            updateTarget();
        }


	}
	
	// Update is called once per frame
	void Update () {
        if (currentMooTimer <= 0) {
            playMoo(false);
            currentMooTimer = mooTimer + Random.Range(-mooVariance, mooVariance);
            audioPlaying = true;
        } else {
            currentMooTimer -= Time.deltaTime;
            if (audioPlaying && source.isPlaying == false) {
                audioPlaying = false;
                if (LevelMode.CURRENT != null) {
                    LevelMode.CURRENT.removeMoo();
                }
            }
        }
        if (LevelMode.CURRENT == null) {
            return;
        }

        if (LevelMode.InSetMode())
        {
            agent.enabled = false;
            return;
        }
        float distanceCheck = Vector3.Distance(target.transform.position, lastTargetPos);
        if ( agent.enabled==true && distanceCheck > updateTargetDistance)
        {
            updateTarget();
        }
        else {
           // Debug.Log("Distance:" + distanceCheck);
        }


	}

    private void updateTarget()
    {
        agent.SetDestination(target.transform.position);
        lastTargetPos = target.transform.position;
    }

    public void springJump(Transform springPos)
    {
        Debug.Log("SPRING SPRING!");
        characterController.enabled = false;
        agent.enabled = false;

        Rigidbody rb = gameObject.GetComponent<Rigidbody>();
        if (rb == null)
        {
            rb = gameObject.AddComponent<Rigidbody>();
        }
        Vector3 mePlus = transform.position;
        mePlus.y += 7.8f;
        Vector3 dif = mePlus - springPos.position;
        rb.velocity = dif * springDeathFactor;
        animator.SetTrigger("SPRING_DEATH");

        playMoo(false);
    }

    public void die(string d)
    {
        Debug.Log("DIED DIED DIED:" + d);
        LevelMode.CURRENT.onEnemyKill();
            
        Destroy(this.gameObject);
    }

    public void playMoo(bool force) {
        if (force || LevelMode.CURRENT==null || (LevelMode.CURRENT!=null && LevelMode.CURRENT.addMoo())) {
            source.clip = mooSounds[Random.Range(0,mooSounds.Length - 1)];
            source.pitch = Random.Range(0.8f, 1.2f);
            source.volume = 0.4f;
            source.Play();
            audioPlaying = true;
        }
    }
}
